/*
	Copyright 2001 Gunter Winkler
*/

#include "fastrand.h"

void sgennormal(PMTDATA& handle, unsigned long seed)
/*
	initialize generator with any nonzero integer
*/
{
	_FR_SRANDOM(handle, seed);
}

void freegennormal(PMTDATA& handle)
/*
	free handle for gennormal
*/
{
	_FR_FREERANDOM(handle);
}

void gennormal(PMTDATA handle, double & y1, double & y2)
{
/*  returns 2 independet normal distributed values
    with mean 0 and variance 1.
 */
/*
	The usual Box-Mller transformation:
         y1 = sqrt( - 2 ln(x1) ) cos( 2 pi x2 )
         y2 = sqrt( - 2 ln(x1) ) sin( 2 pi x2 )
 */
/* a faster version */
         double w;
 
         do {
                 y1 = 2.0 * _FR_RANDOM - 1.0;
                 y2 = 2.0 * _FR_RANDOM - 1.0;
                 w = y1 * y1 + y2 * y2;
         } while ( w >= 1.0 );

         w = sqrt( (-2.0 * log( w ) ) / w );
         y1 *= w;
         y2 *= w;
}
	 
void gencorrnormal(PMTDATA handle, double & y1, double & y2, 
		const double rho, const double sqrtrho )
{
/*  returns 2 correlated normal distributed values
    with mean 0 and variance 1.
	parameters: rho ....... correlation
	            sqrtrho ... must be sqrt(1 - rho^2)
 */
	gennormal(handle, y1, y2);
	y1 = sqrtrho * y1 + rho * y2;
}

void gencorrnormal(PMTDATA handle, double & y1, double & y2, 
		const double rho )
{
	gencorrnormal(handle,y1,y2,rho,sqrt(1-rho*rho));
}
