/*
	Copyright 2001 Gunter Winkler
*/

#include <iostream>
#include "otypes.h"
#include "fastrand.h"
#include "gbm.h"
#include "normal.h"
#include "heston.h"
#include "hestonanalytic.h"

using std::cout;
using std::endl;

static inline double sqr(double a) { return a*a; }

int main(int argc, char* argv[])
{
	long COUNT = 10000;
	
	double dt = 0.00025;
	int steps = 1000;
	
	long i,anz,count;
	double x1,x2;
	double sum1, sum2, sqr1, sqr2, cor;
	double strike,profit,sigma,barrier,gamma,v0;
	
	int hit;
	
	double spot,variance;
	MARKETDATA market;
		
	PMTDATA handle;
	
	sgennormal(handle,429145);
// 	cout.setf(std::ios::fixed, std::ios::floatfield);
 	cout.precision(6);
// 	cout.width(10);	
	
	market.rd     = log(1.06); 
	market.rf     = log(1.03); 	/* (logarithmic) domestic/foreign interest rates */
	market.rho    = -0.5; 
	market.kappa  = 4.5; 
	market.theta  = 0.04; 
	market.xi     = 1.5;	/* stochastic volatility parameters */
	market.lambda = 0.0; 	/* price of volatility risk */

	spot = 1.0;
	strike = 1.0;
	barrier = exp(-0.05);
	v0 = 0.05225;
	sigma = sqrt(v0);
	gamma = exp(-market.rd*dt*steps);

	/**********************/
	
	cout << "rd: " << market.rd;
	cout << ", rf: " << market.rf;
	cout << ", " << endl << "rho: " << market.rho;
	cout << ", kappa: " << market.kappa;
	cout << ", theta: " << market.theta;
	cout << ", xi: " << market.xi;
	cout << ", lambda: " << market.lambda;

	cout << ", " << endl << "spot: " << spot;
	cout << ", strike: " << strike;
	cout << ", barrier: " << barrier;
	cout << ", v_0: " << v0;
	cout << ", time: " << dt*steps;
	cout << endl;

	cout << " analytic BS-Call: "
		 << black_scholes_log( log(spot/strike), strike, dt*steps, sigma,
			                   market.rd, market.rf) << endl;
	cout << " analytic Heston-Call: " 
		 << Heston_Call(spot, v0, dt*steps, strike, market) << endl;
	cout << endl;
	
	cout << " analytic BS-Call (downout): "
		 << black_scholes_downout( log(spot/strike), strike, dt*steps, sigma,
			                   market.rd, market.rf, barrier) << endl;

	cout << " number of sample paths: " << COUNT << endl;
		
	/**********************/
	
	sum1 = 0.0; 
	sum2 = 0.0; 
	sqr1 = 0.0;
	sqr2 = 0.0;
	cor  = 0.0;
	anz = 0;
	
	profit = 0.0;

	for (i=0; i<COUNT; ++i)
	{
		x1 = 1.0;
		hit = gbm_barrier(handle,market.rd-market.rf,sigma,x1,dt,steps,barrier);
		if (-1==hit)
		{
			sum1 += x1;
			sqr1 += x1*x1;
			++anz;
			if (x1>strike) profit += x1-strike;
		}
	}
	x1 = (sum1/anz);
	cout << "knock-out rate: " << ((double)anz/COUNT) << endl;
	cout << "E x1 = " << (x1) << endl;
	cout << "D x1 = " << (sqr1/(anz-1) - sqr(x1)) << endl;
	cout << "E (S-K)+ = " << (gamma*profit/COUNT) << endl;
	cout << endl;
	
	/**********************/
	
	sum1 = 0.0; 
	sum2 = 0.0; 
	sqr1 = 0.0;
	sqr2 = 0.0;
	cor  = 0.0;
	profit = 0.0;
	anz = 0;
	count = 0;
	
	while (true)
	{
		count += COUNT;
		for (i=0; i<COUNT; ++i)
		{
			spot = 1.0;
			variance = v0;
			hit = heston_barrier(handle, spot, variance, market, dt, steps, barrier);
			if (-1==hit)
			{
				sum1 += spot;
				sum2 += variance;
				sqr1 += spot*spot;
				sqr2 += variance*variance;
				++anz;
				if (spot>strike) profit += spot-strike;
			}
		}
		x1 = (sum1/anz);
		x2 = (sum2/anz);
		cout << "knock-out rate: " << ((double)anz/count) << endl;
		cout << "E x = " << (x1) << endl;
		cout << "E v = " << (x2) << endl;
		cout << "D x = " << ((sqr1+sqr(x1))/anz) << endl;
		cout << "D v = " << ((sqr2+sqr(x2))/anz) << endl;
		cout << "E (S-K)+ = " << (gamma*profit/count) << endl;
		cout << endl;
	} // while (true)

	/**********************/
	
	freegennormal(handle);
	
	return 0;
}
